/***************************** matrix_io_d.c *****************************
*	Double Precision IO SUBROUTINES (mex independent)
*	write_xxx	binary to file
*	fprint_xxx	ascii to file.ext (or file if !ext)
*	fscan_xxx	ascii from file
*	read_xxx	binary from file
*	USES:  sprintf(), fwrite(), fopen(), fclose(), fflush(), etc
*	Jeff Fessler
*	COPYRIGHT 1989,  THE BOARD OF TRUSTEES OF THE
*	LELAND STANFORD JUNIOR UNIVERSITY.  ALL RIGHTS RESERVED.
*************************************************************************/
#include "defs-vspline.h"

#define TYPE	double

/*
*	mat[1..N][1..M]
*/
void print_matrix_d(TYPE **mat, int N, int M)
{
	int n;
	for (n=1; n <= N; n++)
		print_vector_d(mat[n], M);
}


/*
*	vect[1..N]
*/
bool write_vector_d(char *name, TYPE *vect, int N)
{
	FILE *fp;
	Fopen0w(fp, name)
	Fwrite0(&vect[1], sizeof(*vect), N, fp)
	Fclose0(fp)
	Ok
}


/*
*	mat[1..N][1..M] -> file "name"(.ext)
*/
bool fprint_matrix_d(char *name, int *ext, TYPE **mat, int N, int M)
{
	int n, m;
	FILE *fp;
	char fname[256];

	if (ext)	sprintf(fname, "%s.%02d", name, *ext);
	else		sprintf(fname, "%s", name);

	Fopen0w(fp, fname)
	for (n=1; n <= N; n++)
	{
		for (m=1; m <= M; m++)
			if (EOF == fprintf(fp, "%g ", mat[n][m]))
				Fail("fprintf")
		if (EOF == fprintf(fp, "\n"))
			Fail("fprintf")
	}
	Fclose0(fp)
	Ok
}


/*
*	vect[1..N] from file "name"
*/

bool fscan_vector_d(TYPE *vect, char *name, int N)
{
	int n;
	FILE *fp;
	Fopen0r(fp, name)
	for (n=1; n <= N; n++)
		if (1 != fscanf(fp, "%lg", &vect[n]))	Fail("fscanf")
	Fclose0(fp)
	Ok
}


/*
*	vect[1..N] from file "name" (binary)
*/

bool read_vector_d(TYPE *vect, char *name, int N)
{
	FILE *fp;

	Fopen0r(fp, name)
	Fread0(&vect[1], sizeof(*vect), N, fp)
	Fclose0(fp)
	Ok
}
